// Inspired by
// https://observablehq.com/@d3/parallel-coordinates
// https://observablehq.com/@d3/brushable-parallel-coordinates

import {
  select,
  transition,
  scaleLinear,
  scalePoint,
  scaleOrdinal,
  schemeCategory10,
  extent,
  line,
  brushY,
} from 'd3';
import { observeResize } from '@curran/responsive-axes';
import { parallelCoordinates } from './parallelCoordinates';
import data from './data.csv';

for (const d of data) {
  d.Total = +d.Total;
  d.HP = +d.HP;
  d.Attack = +d.Attack;
  d.Defense = +d.Defense;
  d['Sp. Atk'] = +d['Sp. Atk'];
  d['Sp. Def'] = +d['Sp. Def'];
  d.Speed = +d.Speed;
  d.Generation = +d.Generation;
}

// Assign d.id to the index for object constancy in transitions.
data.forEach((d, i) => {
  d.id = i;
});

const columns = [
  'Total',
  'HP',
  'Attack',
  'Defense',
  'Sp. Atk',
  'Sp. Def',
  'Speed',
  'Generation',
];

const columnTypes = {
  Total: 'quantitative',
  HP: 'quantitative',
  Attack: 'quantitative',
  Defense: 'quantitative',
  'Sp. Atk': 'quantitative',
  'Sp. Def': 'quantitative',
  Speed: 'quantitative',
  Generation: 'categorical', // Depending on your analysis, this could be 'quantitative'
};

const colorValue = (d) => d.Generation;
const idValue = (d) => d.id;

export const main = (container, { state, setState }) => {
  // Initialize dimensions by observing resise.
  const dimensions = observeResize({
    state,
    setState,
    container,
  });
  if (dimensions === null) return;
  const { width, height } = dimensions;

  // Initialize the brushed intervals state to an empty object.
  if (!state.brushedIntervals) {
    setState((state) => ({
      ...state,
      brushedIntervals: {},
    }));
    return;
  }
  const { brushedIntervals } = state;

  // Updates the brushed intervals when brushes change.
  const updateBrushedInterval = ({ column, interval }) => {
    setState((state) => ({
      ...state,
      brushedIntervals: {
        ...state.brushedIntervals,
        [column]: interval,
      },
    }));
  };

  // Use this to inspect the brushedIntervals as they change!
  // console.log(
  //   JSON.stringify(brushedIntervals, null, 2),
  // );

  // Set up the SVG.
  const svg = select(container)
    .selectAll('svg')
    .data([null])
    .join('svg')
    .attr('width', width)
    .attr('height', height)
    .style('background', '#090F10');

  // Invoke the parallel coordinates.
  svg.call(parallelCoordinates, {
    data,
    columns,
    columnTypes,
    colorValue,
    idValue,
    width,
    height,
    brushedIntervals,
    updateBrushedInterval,
  });
};
